"""
QuackOSM.

QuackOSM is a Python library used for reading pbf (ProtoBuffer) files with OpenStreetMap data using
DuckDB spatial extension without GDAL.
"""

from quackosm.functions import (
    convert_geometry_to_gpq,
    convert_pbf_to_gpq,
    get_features_gdf,
    get_features_gdf_from_geometry,
)
from quackosm.pbf_file_reader import PbfFileReader

__app_name__ = "QuackOSM"
__version__ = "0.5.2"

__all__ = [
    "PbfFileReader",
    "convert_pbf_to_gpq",
    "convert_geometry_to_gpq",
    "get_features_gdf",
    "get_features_gdf_from_geometry",
]



"""
Geofabrik OpenStreetMap extracts.

This module contains wrapper for publically available Geofabrik download server.
"""

import json
import operator
from pathlib import Path
from typing import Optional

import geopandas as gpd
import requests

GEOFABRIK_INDEX_URL = "https://download.geofabrik.de/index-v1.json"
GEOFABRIK_INDEX_GDF: Optional[gpd.GeoDataFrame] = None

__all__ = ["_get_geofabrik_index"]


def _get_geofabrik_index() -> gpd.GeoDataFrame:
    global GEOFABRIK_INDEX_GDF  # noqa: PLW0603

    if GEOFABRIK_INDEX_GDF is None:
        GEOFABRIK_INDEX_GDF = _load_geofabrik_index()

    return GEOFABRIK_INDEX_GDF


def _load_geofabrik_index() -> gpd.GeoDataFrame:
    """
    Load available extracts from GeoFabrik download service.

    Returns:
        gpd.GeoDataFrame: Extracts index with metadata.
    """
    save_path = Path("cache/geofabrik_index.geojson")

    if save_path.exists():
        gdf = gpd.read_file(save_path)
    else:
        result = requests.get(
            GEOFABRIK_INDEX_URL,
            headers={
                "User-Agent": "QuackOSM Python package (https://github.com/kraina-ai/quackosm)"
            },
        )
        parsed_data = json.loads(result.text)
        gdf = gpd.GeoDataFrame.from_features(parsed_data["features"])
        gdf["area"] = gdf.geometry.area
        gdf.sort_values(by="area", ignore_index=True, inplace=True)
        gdf["url"] = gdf["urls"].apply(operator.itemgetter("pbf"))
        gdf["id"] = "Geofabrik_" + gdf["id"]
        gdf = gdf[["id", "name", "geometry", "area", "url"]]

        save_path.parent.mkdir(parents=True, exist_ok=True)
        gdf.to_file(save_path, driver="GeoJSON")

    return gdf



"""Main CLI module."""


def main() -> None:
    """Run provided CLI."""
    try:
        from quackosm import __app_name__, cli
    except ImportError as exc:
        error_msg = (
            "Missing optional dependencies required for the CLI."
            " Please install required packages using `pip install quackosm[cli]`."
        )
        raise ImportError(error_msg) from exc

    cli.app(prog_name=__app_name__)


if __name__ == "__main__":
    main()



from collections.abc import Iterable
from typing import Any, NamedTuple, cast

from quackosm._typing import is_expected_type


class OsmWayPolygonConfig(NamedTuple):
    """OSM Way polygon features config object."""

    all: Iterable[str]
    allowlist: dict[str, Iterable[str]]
    denylist: dict[str, Iterable[str]]


def parse_dict_to_config_object(raw_config: dict[str, Any]) -> OsmWayPolygonConfig:
    all_tags = raw_config.get("all", [])
    allowlist_tags = raw_config.get("allowlist", {})
    denylist_tags = raw_config.get("denylist", {})
    if not is_expected_type(all_tags, Iterable[str]):
        raise ValueError(f"Wrong type of key: all ({type(all_tags)})")

    if not is_expected_type(allowlist_tags, dict[str, Iterable[str]]):
        raise ValueError(f"Wrong type of key: all ({type(allowlist_tags)})")

    if not is_expected_type(denylist_tags, dict[str, Iterable[str]]):
        raise ValueError(f"Wrong type of key: denylist ({type(denylist_tags)})")

    return OsmWayPolygonConfig(
        all=cast(Iterable[str], all_tags),
        allowlist=cast(dict[str, Iterable[str]], allowlist_tags),
        denylist=cast(dict[str, Iterable[str]], denylist_tags),
    )
